@@ -1,17 +1,28 @@
 package disk
 
 import (
+	"errors"
 	"fmt"
 	"runtime"
+	"sync"
 	"testing"
+
+	"github.com/shirou/gopsutil/v3/internal/common"
 )
 
+func skipIfNotImplementedErr(t *testing.T, err error) {
+	if errors.Is(err, common.ErrNotImplementedError) {
+		t.Skip("not implemented")
+	}
+}
+
 func TestDisk_usage(t *testing.T) {
 	path := "/"
 	if runtime.GOOS == "windows" {
 		path = "C:"
 	}
 	v, err := Usage(path)
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -22,17 +33,17 @@ func TestDisk_usage(t *testing.T) {
 
 func TestDisk_partitions(t *testing.T) {
 	ret, err := Partitions(false)
+	skipIfNotImplementedErr(t, err)
 	if err != nil || len(ret) == 0 {
 		t.Errorf("error %v", err)
 	}
 	t.Log(ret)
 
-	empty := PartitionStat{}
 	if len(ret) == 0 {
 		t.Errorf("ret is empty")
 	}
 	for _, disk := range ret {
-		if disk == empty {
+		if disk.Device == "" {
 			t.Errorf("Could not get device info %v", disk)
 		}
 	}
@@ -40,6 +51,7 @@ func TestDisk_partitions(t *testing.T) {
 
 func TestDisk_io_counters(t *testing.T) {
 	ret, err := IOCounters()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -55,6 +67,23 @@ func TestDisk_io_counters(t *testing.T) {
 	}
 }
 
+// https://github.com/shirou/gopsutil/issues/560 regression test
+func TestDisk_io_counters_concurrency_on_darwin_cgo(t *testing.T) {
+	if runtime.GOOS != "darwin" {
+		t.Skip("darwin only")
+	}
+	var wg sync.WaitGroup
+	const max = 1000
+	for i := 1; i < max; i++ {
+		wg.Add(1)
+		go func() {
+			defer wg.Done()
+			IOCounters()
+		}()
+	}
+	wg.Wait()
+}
+
 func TestDiskUsageStat_String(t *testing.T) {
 	v := UsageStat{
 		Path:              "/",
@@ -79,9 +108,9 @@ func TestDiskPartitionStat_String(t *testing.T) {
 		Device:     "sd01",
 		Mountpoint: "/",
 		Fstype:     "ext4",
-		Opts:       "ro",
+		Opts:       []string{"ro"},
 	}
-	e := `{"device":"sd01","mountpoint":"/","fstype":"ext4","opts":"ro"}`
+	e := `{"device":"sd01","mountpoint":"/","fstype":"ext4","opts":["ro"]}`
 	if e != fmt.Sprintf("%v", v) {
 		t.Errorf("DiskUsageStat string is invalid: %v", v)
 	}
