@@ -1,13 +1,24 @@
 package host
 
 import (
+	"errors"
 	"fmt"
 	"os"
+	"sync"
 	"testing"
+
+	"github.com/shirou/gopsutil/v3/internal/common"
 )
 
+func skipIfNotImplementedErr(t *testing.T, err error) {
+	if errors.Is(err, common.ErrNotImplementedError) {
+		t.Skip("not implemented")
+	}
+}
+
 func TestHostInfo(t *testing.T) {
 	v, err := Info()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -26,6 +37,7 @@ func TestUptime(t *testing.T) {
 	}
 
 	v, err := Uptime()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -39,6 +51,7 @@ func TestBoot_time(t *testing.T) {
 		t.Skip("Skip CI")
 	}
 	v, err := BootTime()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -51,6 +64,7 @@ func TestBoot_time(t *testing.T) {
 	t.Logf("first boot time: %d", v)
 
 	v2, err := BootTime()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
@@ -62,12 +76,13 @@ func TestBoot_time(t *testing.T) {
 
 func TestUsers(t *testing.T) {
 	v, err := Users()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("error %v", err)
 	}
 	empty := UserStat{}
 	if len(v) == 0 {
-		t.Fatal("Users is empty")
+		t.Skip("Users is empty")
 	}
 	for _, u := range v {
 		if u == empty {
@@ -78,17 +93,18 @@ func TestUsers(t *testing.T) {
 
 func TestHostInfoStat_String(t *testing.T) {
 	v := InfoStat{
-		Hostname: "test",
-		Uptime:   3000,
-		Procs:    100,
-		OS:       "linux",
-		Platform: "ubuntu",
-		BootTime: 1447040000,
-		HostID:   "edfd25ff-3c9c-b1a4-e660-bd826495ad35",
+		Hostname:   "test",
+		Uptime:     3000,
+		Procs:      100,
+		OS:         "linux",
+		Platform:   "ubuntu",
+		BootTime:   1447040000,
+		HostID:     "edfd25ff-3c9c-b1a4-e660-bd826495ad35",
+		KernelArch: "x86_64",
 	}
-	e := `{"hostname":"test","uptime":3000,"bootTime":1447040000,"procs":100,"os":"linux","platform":"ubuntu","platformFamily":"","platformVersion":"","kernelVersion":"","virtualizationSystem":"","virtualizationRole":"","hostid":"edfd25ff-3c9c-b1a4-e660-bd826495ad35"}`
+	e := `{"hostname":"test","uptime":3000,"bootTime":1447040000,"procs":100,"os":"linux","platform":"ubuntu","platformFamily":"","platformVersion":"","kernelVersion":"","kernelArch":"x86_64","virtualizationSystem":"","virtualizationRole":"","hostId":"edfd25ff-3c9c-b1a4-e660-bd826495ad35"}`
 	if e != fmt.Sprintf("%v", v) {
-		t.Errorf("HostInfoStat string is invalid: %v", v)
+		t.Errorf("HostInfoStat string is invalid:\ngot  %v\nwant %v", v, e)
 	}
 }
 
@@ -106,14 +122,15 @@ func TestUserStat_String(t *testing.T) {
 }
 
 func TestHostGuid(t *testing.T) {
-	hi, err := Info()
+	id, err := HostID()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Error(err)
 	}
-	if hi.HostID == "" {
+	if id == "" {
 		t.Error("Host id is empty")
 	} else {
-		t.Logf("Host id value: %v", hi.HostID)
+		t.Logf("Host id value: %v", id)
 	}
 }
 
@@ -121,32 +138,44 @@ func TestTemperatureStat_String(t *testing.T) {
 	v := TemperatureStat{
 		SensorKey:   "CPU",
 		Temperature: 1.1,
+		High:        30.1,
+		Critical:    0.1,
 	}
-	s := `{"sensorKey":"CPU","sensorTemperature":1.1}`
+	s := `{"sensorKey":"CPU","temperature":1.1,"sensorHigh":30.1,"sensorCritical":0.1}`
 	if s != fmt.Sprintf("%v", v) {
-		t.Errorf("TemperatureStat string is invalid")
+		t.Errorf("TemperatureStat string is invalid, %v", fmt.Sprintf("%v", v))
 	}
 }
 
 func TestVirtualization(t *testing.T) {
-	system, role, err := Virtualization()
-	if err != nil {
-		t.Errorf("Virtualization() failed, %v", err)
-	}
-	if system == "" || role == "" {
-		t.Errorf("Virtualization() retuns empty system or role:  %s, %s", system, role)
-	}
+	wg := sync.WaitGroup{}
+	testCount := 10
+	wg.Add(testCount)
+	for i := 0; i < testCount; i++ {
+		go func(j int) {
+			system, role, err := Virtualization()
+			wg.Done()
+			skipIfNotImplementedErr(t, err)
+			if err != nil {
+				t.Errorf("Virtualization() failed, %v", err)
+			}
 
-	t.Logf("Virtualization(): %s, %s", system, role)
+			if j == 9 {
+				t.Logf("Virtualization(): %s, %s", system, role)
+			}
+		}(i)
+	}
+	wg.Wait()
 }
 
 func TestKernelVersion(t *testing.T) {
 	version, err := KernelVersion()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("KernelVersion() failed, %v", err)
 	}
 	if version == "" {
-		t.Errorf("KernelVersion() retuns empty: %s", version)
+		t.Errorf("KernelVersion() returns empty: %s", version)
 	}
 
 	t.Logf("KernelVersion(): %s", version)
@@ -154,11 +183,12 @@ func TestKernelVersion(t *testing.T) {
 
 func TestPlatformInformation(t *testing.T) {
 	platform, family, version, err := PlatformInformation()
+	skipIfNotImplementedErr(t, err)
 	if err != nil {
 		t.Errorf("PlatformInformation() failed, %v", err)
 	}
 	if platform == "" {
-		t.Errorf("PlatformInformation() retuns empty: %v", platform)
+		t.Errorf("PlatformInformation() returns empty: %v", platform)
 	}
 
 	t.Logf("PlatformInformation(): %v, %v, %v", platform, family, version)
